#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <time.h>
#include <string.h>

#include "thread/thread.h"
#include "httpp/httpp.h"

#include "connection.h"
#include "refbuf.h"
#include "client.h"

#include "os.h"
#include "cfgfile.h"
#include "logging.h"

#ifdef _WIN32
#define snprintf _snprintf
#endif

/* the global log descriptors */
int errorlog = 0;
int accesslog = 0;


char lastFilePostfix[9] = "";

/* 
** ADDR USER AUTH DATE REQUEST CODE BYTES REFERER AGENT [TIME]
**
** ADDR = client->con->ip
** USER = -      
**      we should do this for real once we support authentication
** AUTH = -
** DATE = _make_date(client->con->con_time)
** REQUEST = build from client->parser
** CODE = client->respcode
** BYTES = client->con->sent_bytes
** REFERER = get from client->parser
** AGENT = get from client->parser
** TIME = timing_get_time() - client->con->con_time
*/
void logging_access(client_t *client)
{
    char datebuf[128];
    char reqbuf[1024];
    struct tm *thetime;
    time_t now;
    now = time(NULL);

    unsigned long longNow = (long) now;

    /* build the data */
    /* TODO: localtime is not threadsafe on all platforms
    ** we should probably use localtime_r if it's available
    */
    PROTECT_CODE(thetime = localtime(&now); 
	strftime(datebuf, 128, LOGGING_FORMAT_CLF, thetime))


    ice_config_t *config = config_get_config_unlocked();

    char newFilePostfix[15];

    getLogfilePostfix(newFilePostfix, thetime, config);


// check if file exists, and if we write to the actual log file

    char filename[FILENAME_MAX];

    getLogfileNameForAggregation(filename, thetime, config);

    int fileExists;
    fileExists = access(filename, F_OK);

    if ( fileExists == 0 && strcmp(lastFilePostfix, newFilePostfix) == 0 ) {

	// log file postfix has not changed, thus DO NOTHING WITH THE LOGFILE

    }
    else {

	// log file "postfix" has changed; generate a new file

        log_close(accesslog);

        accesslog = log_open(filename);

	// set the new file postfix to memorize actual logfile postfix
        strcpy(lastFilePostfix,newFilePostfix);
    }

    /* build the request */
    snprintf(reqbuf, 1024, "%s %s %s/%s", httpp_getvar(client->parser, HTTPP_VAR_REQ_TYPE), httpp_getvar(client->parser, HTTPP_VAR_URI),
         httpp_getvar(client->parser, HTTPP_VAR_PROTOCOL), httpp_getvar(client->parser, HTTPP_VAR_VERSION));

    unsigned long longStayed = longNow - (long) client->con->con_time;

    log_write_direct(accesslog, "%s - - [%s] \"%s\" %d %lld \"%s\" \"%s\" %d",
             client->con->ip,
             datebuf,
             reqbuf,
             client->respcode,
             client->con->sent_bytes,
             (httpp_getvar(client->parser, "referer") != NULL) ? httpp_getvar(client->parser, "referer") : "-",
             (httpp_getvar(client->parser, "user-agent") != NULL) ? httpp_getvar(client->parser, "user-agent") : "-",
             longStayed);
}


void restart_logging ()
{
    printf("HIER  SAMMA JETZT ABER");

    ice_config_t *config = config_get_config_unlocked();

    if (strcmp (config->error_log, "-"))
    {
        char fn_error[FILENAME_MAX];
        snprintf (fn_error, FILENAME_MAX, "%s%s%s", config->log_dir, PATH_SEPARATOR, config->error_log);
        log_set_filename (errorlog, fn_error);
        log_set_level (errorlog, config->loglevel);
        log_reopen (errorlog);
    }

    if (strcmp (config->access_log, "-"))
    {
        char fn_error[FILENAME_MAX];
        snprintf (fn_error, FILENAME_MAX, "%s%s%s", config->log_dir, PATH_SEPARATOR, config->access_log);
        log_set_filename (accesslog, fn_error);
        log_reopen (accesslog);
    }
}

/*
**   sets newFilePostfix, build from the given tm struct, and the
**   corresponding aggregation mode from ice_config_t
*/

void getLogfilePostfix(char newFilePostfix[], struct tm *thetime, ice_config_t *config) {

    char year[5];
    char month[3];
    char day[3];

    char hours[]   = "00";
    char minutes[] = "00";
    char secs[]    = "00";


    switch(config->aggregation) {
					// no aggregation
        case 0: 
                strcpy(newFilePostfix, "");		
		return;
	
					// daily
	case 1: 
		sprintf(year, "%04d", thetime->tm_year+1900);
		sprintf(month, "%02d", thetime->tm_mon+1);
		sprintf(day, "%02d", thetime->tm_mday);

		break;
					// monthly
	case 2:

		sprintf(year, "%04d", thetime->tm_year+1900);
		sprintf(month, "%02d", thetime->tm_mon+1);
		strcpy(day, "01");
		break;
					// just break;
	default:
		strcpy(newFilePostfix, "");
		return;
 
    }

    snprintf (newFilePostfix, 16, ".%s%s%s%s%s%s", year, month, day, hours, minutes, secs);
}

/*
**   sets given filename, build from the given tm struct, and the
**   corresponding aggregation mode from ice_config_t
*/

void getLogfileNameForAggregation(char filename[], struct tm *thetime, ice_config_t *config) {

    char newFilePostfix[16];

    getLogfilePostfix(newFilePostfix, thetime, config);

    snprintf (filename, FILENAME_MAX, "%s%s%s", config->log_dir, PATH_SEPARATOR, config->access_log);

    strcat(filename, newFilePostfix);
}
